/**
 * @file ladel_row_mod.h
 * @author Ben Hermans
 * @brief Routines for the row add/delete updates to a factorization are defined in ladel_row_mod.c. 
 */

#ifndef LADEL_ROW_MOD_H
#define LADEL_ROW_MOD_H

#ifdef __cplusplus
extern "C" {
#endif 

#include "ladel_types.h"

#ifdef __cplusplus
}
#endif 

#endif /*LADEL_ROW_MOD_H*/